/*
 * nasd_keymgmt_afs.c
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Howard Gobioff
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

/***********************************************************************
 *
 * nasd_keymgmt.c
 *
 * Routines to access keys 
 * Use by both the file manager and the drive
 *
 ***********************************************************************/
#define PRINT_ERROR 1

#ifdef DEBUG_ALL
#define DEBUG_INIT_KEY_MGMT 1
#define DEBUG_KEYS
#endif

#include <nasd/nasd_options.h>
#include <nasd/nasd_keymgmt_afs.h>
#include <nasd/nasd_threadstuff.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_mem.h>

/* This is a hacked module that has one set of keys for everyone --
the sole purpose is to get the demo up and running */



NASD_DECLARE_MUTEX(keylock)


nasd_key_t red_key;
nasd_key_t black_key;
nasd_key_t partition_key;

/* Our lock on the key hiearchy - grab it when you
   change or read a lock*/



nasd_status_t nasd_security_init_key_mgmt(char *keyfile,char *password,int init_from_key){

#if NASD_SECURE_RPCS_ENABLE > 0
 
  nasd_partnum_t partnum;
  int i;
#if  DEBUG_INIT_KEY_MGMT > 0
  printf("Initializing Keylock \n");
#endif
  nasd_mutex_init(&keylock);
  
  partnum=1;
  for(i=0;i<sizeof(nasd_key_t);i++) {
    
    partition_key[i]=partnum;
    red_key[i]=partnum+1;
    black_key[i]=partnum+2;
  }

  return NASD_SUCCESS;
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
}

nasd_status_t nasd_security_get_red_key(int foo, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy(red_key,target,sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}


nasd_status_t nasd_security_get_black_key(int foo, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy(black_key,target,sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}


nasd_status_t nasd_security_get_partition_key(nasd_partnum_t partnum, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy(partition_key,target,sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}





