/*
 * nasd_cheops_cl_noraid.c
 */
/*
 * NORAID access module
 *
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Khalil Amiri, CMU SCS/ECE, July 18 1997
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
#include <nasd/nasd_options.h>
#include <sys/errno.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_cheops_types.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_cheops_types.h>
#include <nasd/nasd_cheops_common.h>
#include <nasd/nasd_cheops_raidmap.h>
#include <nasd/nasd_cheops_cache.h>
#include <nasd/nasd_cheops_ios.h>
/*
 * XXX this should be fixed. It will not be correctly included
 * as a dependency on many platforms. Also, the relative path
 * thing makes it hard for alternatively-organized trees, and
 * just give up on including something like this in the kernel.
 */
#include "../cheops/shared/nasd_cheops_cl_drive.h"

extern void _nasd_cheops_asm_freeall(nasd_asm_t *);
extern void _nasd_cheops_io_freeall(_nasd_cheops_aggr_io_t *);
extern void _nasd_cheops_init_io(NASD_pda_t *, _nasd_cheops_aggr_io_t *, _nasd_cheops_io_t *);
extern void _nasd_cheops_enq_io(_nasd_cheops_io_t *);

extern _nasd_cheops_cache_t *_nasd_cheops_dr_handle_cachep;

int
_nasd_cheops_noraid_read(nasd_asm_t *asm, nasd_len_t *out_len)
{
        int rc = 0;
	NASD_pda_t *pda;
	nasd_identifier_t ni;
	nasd_disk_ident_t di;
	nasd_uint64 bms;
	nasd_status_t nasd_status;
	nasd_rpc_status_t status;
	nasd_key_t req_key;
	nasd_partnum_t partnum;
	handle_t h;
	_nasd_cheops_cache_key_t in_key;
	_nasd_cheops_dr_handle_t dr_handle;

	if (asm==NULL)
	  return EINVAL;	

	pda = &asm->first_stripe_access->pda_list[0];

	if (pda->buf == NULL)
	  return EINVAL;

	ni = pda->data_obj_id.nasd_identifier;
	di = pda->data_obj_id.disk_identifier;

        /* lookup a handle for the target drive */
	partnum = 0;
	
	/* FIX -- get cookie from pda */
	bms = 0;
	rc = _nasd_cheops_read_dr(di, partnum, req_key, ni,
				  pda->buf, pda->start_address, pda->len, bms,
				  out_len, &nasd_status, &status);

	/*(cheops_Msg "in noraid read \n");*/
	_nasd_cheops_asm_freeall(asm);

	if (rc)
	  return rc;

	return 0;
}

int
_nasd_cheops_noraid_write(nasd_asm_t *asm, nasd_len_t *out_len)
{
        int rc = 0;
	NASD_pda_t *pda;
	nasd_identifier_t ni;
	nasd_disk_ident_t di;
	nasd_status_t nasd_status;
	nasd_rpc_status_t status;
	nasd_key_t req_key;
	nasd_partnum_t partnum;
	handle_t h;
	_nasd_cheops_cache_key_t in_key;
	_nasd_cheops_dr_handle_t dr_handle;

	if (asm==NULL)
	  return EINVAL;	

	pda = &asm->first_stripe_access->pda_list[0];

	if (pda->buf == NULL )
	  return EINVAL;

	ni = pda->data_obj_id.nasd_identifier;
	di = pda->data_obj_id.disk_identifier;

        /* lookup a handle for the target drive */
	partnum = 0;	

	rc = _nasd_cheops_write_dr(di, partnum, req_key, ni,
				  pda->buf, pda->start_address, pda->len,
				  out_len, &nasd_status, &status);

	/*(cheops_Msg "in noraid write \n");*/
	_nasd_cheops_asm_freeall(asm);

	if (rc)
	  return rc;

	return 0;
}



