/*
 * nasd_alpha_incstat.s
 *
 * Increment nasd statistic word
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <machine/asm.h>
#include <machine/regdef.h>

/*
 * Atomically increment a 64-bit word.
 * Load it locked, increment it, store it.
 * The logical thing might seem to be load,
 * align, inc, store, beq to beginning to try again.
 * The Alpha AXP arch ref man suggests not
 * doing this because the processor predicts
 * backward branch taken, so instead we do a bne
 * forward-then-backward (we optimistically assume
 * that conflict is unlikely, which seems right).
 *
 * This happens to return the old value in case
 * anyone cares.
 */
LEAF(nasd_atomic_inc64)
1:
	ldq_l	v0,(a0)
	bis	zero,zero,zero /* magic to align subsequent instructions */
	addq	v0,1,t0
	stq_c	t0,(a0)
	beq	t0,2f
	ret	zero,(ra)
2:
	br	zero,1b
END(nasd_atomic_inc64)

/*
 * Same as atomic_inc64, except instead of adding 1, we
 * add the contents of a1 (next argument).
 */
LEAF(nasd_atomic_add64)
1:
	ldq_l	v0,(a0)
	bis	zero,zero,zero /* magic to align subsequent instructions */
	addq	v0,a1,t0
	stq_c	t0,(a0)
	beq	t0,2f
	ret	zero,(ra)
2:
	br	zero,1b
END(nasd_atomic_add64)

/*
 * Same as atomic_add64, except we subtract a1 (next argument)
 * instead of adding it.
 */
LEAF(nasd_atomic_sub64)
1:
	ldq_l	v0,(a0)
	bis	zero,zero,zero /* magic to align subsequent instructions */
	subq	v0,a1,t0
	stq_c	t0,(a0)
	beq	t0,2f
	ret	zero,(ra)
2:
	br	zero,1b
END(nasd_atomic_sub64)
