/*
 * nasd_generic_incstat.c
 *
 * Generic inc/add/subtract nasd statistic word for most OSes.
 * Not very fast (involves locking/unlocking a mutex), but is portable.
 * If your architecture has atomic inc/add/sub operations, you probably
 * want to write a machine-specific version of this file.
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Marc Unangst
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_threadstuff.h>

NASD_DECLARE_EXTERN_MUTEX(nasd_atomic_global_mutex);

/* Atomically increment an integer. */
void
nasd_atomic_inc64(nasd_int64 *i)
{
  NASD_LOCK_MUTEX(nasd_atomic_global_mutex);
  (*i)++;
  NASD_UNLOCK_MUTEX(nasd_atomic_global_mutex);
}

/* Atomically add one integer to another. */
void
nasd_atomic_add64(nasd_int64 *i, nasd_int64 j)
{
  NASD_LOCK_MUTEX(nasd_atomic_global_mutex);
  *i += j;
  NASD_UNLOCK_MUTEX(nasd_atomic_global_mutex);
}

/* Atomically subtract one integer from another. */
void
nasd_atomic_sub64(nasd_int64 *i, nasd_int64 j)
{
  NASD_LOCK_MUTEX(nasd_atomic_global_mutex);
  *i -= j;
  NASD_UNLOCK_MUTEX(nasd_atomic_global_mutex);
}
