/*
 * nasd_linux_basic.c
 *
 * Basic system support stuff
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: Marc Unangst, Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_threadstuff.h>
#include <nasd/nasd_timer.h>
#include <nasd/nasd_common.h>
#include <sys/time.h>
#include <fcntl.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <ctype.h>

nasd_uint64 nasd_ctimer_ticks;

nasd_status_t
nasd_sys_init()
{
  nasd_get_clock_speed(&nasd_ctimer_ticks);

  return(NASD_SUCCESS);
}

static nasd_timespec_t nasd_linux_last_time = {0,0};
static nasd_timespec_t nasd_linux_uniq_time = {0,0};

void
nasd_gettime(
  nasd_timespec_t  *tm)
{
  struct timeval tv;

  gettimeofday(&tv, NULL);
  tm->ts_sec = tv.tv_sec;
  tm->ts_nsec = tv.tv_usec*1000;
  if ((tm->ts_sec == nasd_linux_last_time.ts_sec)
    && (tm->ts_nsec == nasd_linux_last_time.ts_nsec))
  {
    nasd_linux_uniq_time.ts_nsec++;
    if (nasd_linux_uniq_time.ts_nsec >= NASD_NSEC_PER_SEC) {
      nasd_linux_uniq_time.ts_nsec -= NASD_NSEC_PER_SEC;
      nasd_linux_uniq_time.ts_sec++;
    }
    *tm = nasd_linux_uniq_time;
  }
  else {
    nasd_linux_last_time = *tm;
    nasd_linux_uniq_time = nasd_linux_last_time;
  }
}

nasd_status_t
nasd_gethostbyname_r(
  char            *hostname,
  struct hostent  *hep)
{
  struct hostent *lhep;
  /* XXXmju - I have no idea if this is an appropriate size, but there
     is no system constant or documented guideline.  It should be "big
     enough." */
  char hd[1024];
  int herr, ret;

  bzero((char *)&hd, sizeof(hd));
#if __GLIBC__
  ret = gethostbyname_r(hostname, hep, hd, sizeof(hd), &lhep, &herr);
  if (ret) {
    return(NASD_FAIL);
  }
#else /* __GLIBC__ */
  lhep = gethostbyname_r(hostname, hep, hd, sizeof(hd), &herr);
  if (lhep == NULL) {
    return(NASD_FAIL);
  }
#endif /* __GLIBC__ */
  return(NASD_SUCCESS);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
