/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

/***********************************************************************
 *
 * nasd_common_otw.c
 *
 * To convert the 64 bit ints (DCE sucks, the OSF sucks, DEC sucks)
 *
 ***********************************************************************/

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_nfs_types.h>

#include <nasd/nasd_pdrive.h> 
#include <nasd/nasd_mem.h>

#define PERFORM_TRICKERY  /* if this is set, then we just point the output at the input */

void nasd_int64_from_xmit(nasd_int64_otw_t *in, nasd_int64 *out)
{
  *out = *((nasd_int64*)in);
}

void nasd_int64_to_xmit(nasd_int64* in, nasd_int64_otw_t **out)
{
#ifdef _KERNEL
  /* XXX - we can't allocate kernel memory (too slow), so we're going to cheat */
  *out = (nasd_int64_otw_t *)in;
#else /* _KERNEL */
#ifdef PERFORM_TRICKERY
  *out = (nasd_int64_otw_t *)in;
#else /* PERFORM_TRICKERY */
  *out = (nasd_int64_otw_t *)malloc(sizeof(nasd_int64_otw_t));
  *((nasd_int64 *)*out) = *in;
#endif /* PERFORM_TRICKERY */
#endif /* _KERNEL */
}

void nasd_int64_free_inst(nasd_int64 *in)
{
  /* nothing to do here */
}

void nasd_int64_free_xmit(nasd_int64_otw_t *in)
{
#ifdef _KERNEL
  /* in the kernel we cheated (see above) */
#else /* _KERNEL */
#ifdef PERFORM_TRICKERY
  /* cheat (see above) */
#else /* PERFORM_TRICKERY */
  free(in);
#endif /* PERFORM_TRICKERY */
#endif /* _KERNEL */
}



void nasd_uint64_from_xmit(nasd_int64_otw_t *in, nasd_uint64 *out)
{
  *out = *((nasd_uint64*)in);
}

void nasd_uint64_to_xmit(nasd_uint64* in, nasd_int64_otw_t **out)
{
  *out = (nasd_int64_otw_t *)in;
}

void nasd_uint64_free_inst(nasd_uint64 *in)
{
  /* nothing to do here */
}

void nasd_uint64_free_xmit(nasd_int64_otw_t *in)
{
  /* nothing to do here */
}



void nasd_identifier_t_from_xmit(nasd_int64_otw_t *in, nasd_identifier_t *out)
{
  *out = *((nasd_identifier_t*)in);
}

void nasd_identifier_t_to_xmit(nasd_identifier_t* in, nasd_int64_otw_t **out)
{
  *out = (nasd_int64_otw_t *)in;
}

void nasd_identifier_t_free_inst(nasd_identifier_t *in)
{
  /* nothing to do here */
}

void nasd_identifier_t_free_xmit(nasd_int64_otw_t *in)
{
  /* nothing to do here */
}


/* The rest of them cheat in the kernel and outside */

void nasd_marker_t_from_xmit(nasd_int64_otw_t *in, nasd_marker_t *out) { *out = *((nasd_marker_t*)in); }
void nasd_marker_t_to_xmit(nasd_marker_t* in, nasd_int64_otw_t **out) { *out = (nasd_int64_otw_t *)in; }
void nasd_marker_t_free_inst(nasd_marker_t *in) { /* nothing to do here */ }
void nasd_marker_t_free_xmit(nasd_int64_otw_t *in) { /* nothing to do */ }

void nasd_markerverf_t_from_xmit(nasd_int64_otw_t *in, nasd_markerverf_t *out) { *out = *((nasd_markerverf_t*)in); }
void nasd_markerverf_t_to_xmit(nasd_markerverf_t* in, nasd_int64_otw_t **out) { *out = (nasd_int64_otw_t *)in; }
void nasd_markerverf_t_free_inst(nasd_markerverf_t *in) { /* nothing to do here */ }
void nasd_markerverf_t_free_xmit(nasd_int64_otw_t *in) { /* nothing to do */ }

void nasd_disk_ident_t_from_xmit(nasd_int64_otw_t *in, nasd_disk_ident_t *out) { *out = *((nasd_disk_ident_t*)in); }
void nasd_disk_ident_t_to_xmit(nasd_disk_ident_t* in, nasd_int64_otw_t **out) { *out = (nasd_int64_otw_t *)in; }
void nasd_disk_ident_t_free_inst(nasd_disk_ident_t *in) { /* nothing to do here */ }
void nasd_disk_ident_t_free_xmit(nasd_int64_otw_t *in) { /* nothing to do */ }

void nasd_offset_t_from_xmit(nasd_int64_otw_t *in, nasd_offset_t *out) { *out = *((nasd_offset_t*)in); }
void nasd_offset_t_to_xmit(nasd_offset_t* in, nasd_int64_otw_t **out) { *out = (nasd_int64_otw_t *)in; }
void nasd_offset_t_free_inst(nasd_offset_t *in) { /* nothing to do here */ }
void nasd_offset_t_free_xmit(nasd_int64_otw_t *in) { /* nothing to do */ }

void nasd_opcount_t_from_xmit(nasd_int64_otw_t *in, nasd_opcount_t *out) { *out = *((nasd_opcount_t*)in); }
void nasd_opcount_t_to_xmit(nasd_opcount_t* in, nasd_int64_otw_t **out) { *out = (nasd_int64_otw_t *)in; }
void nasd_opcount_t_free_inst(nasd_opcount_t *in) { /* nothing to do here */ }
void nasd_opcount_t_free_xmit(nasd_int64_otw_t *in) { /* nothing to do */ }
