/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Howard Gobioff
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

/***********************************************************************
 *
 * nasd_security.c
 *
 * Put the Security in NASD!
 *
 * This modules was temporarily gone. Now it contains the routine
 * to convert a 160 bit credential key into the integrity key and privacy key
 ***********************************************************************/

#include <nasd/nasd_options.h>
#include <nasd/nasd_general.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_drive_types.h>
#include <nasd/nasd_security.h>
#include <cryptolib/libcrypt.h>

#ifndef KERNEL
#include <string.h>
#endif /* !KERNEL */

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/* This should be about the size the internal compression function of
   our hash function takes ... hgobioff */

#define NASD_HASH_BASIC_SIZE SHA_CBLOCK

#define NASD_INTEGRITY_CONSTANT 0xA
#define NASD_PRIVACY_CONSTANT 0x5
nasd_status_t nasd_security_generate_integrity_key(nasd_key_t cred_key,
                                                   nasd_key_t integ_key) {
  unsigned char buf[NASD_HASH_BASIC_SIZE];
  SHA1_CTX context;
  nasd_digest_t tdigest;

  memset(buf, NASD_INTEGRITY_CONSTANT, NASD_HASH_BASIC_SIZE);

  SHA1_Init(FALSE, &context);
  memcpy(buf, cred_key, sizeof(nasd_key_t));
  SHA1_Update(FALSE, &context, buf, NASD_HASH_BASIC_SIZE);
  SHA1_Final(FALSE, tdigest, &context);
  bcopy(tdigest, integ_key, NASD_MIN(sizeof(tdigest), sizeof(nasd_key_t)));

  return (NASD_SUCCESS);
}


nasd_status_t nasd_security_generate_privacy_key(nasd_key_t cred_key,
                                                   nasd_key_t priv_key) {

  unsigned char buf[NASD_HASH_BASIC_SIZE];
  SHA1_CTX context;
  nasd_digest_t tdigest;

  memset(buf, NASD_PRIVACY_CONSTANT, NASD_HASH_BASIC_SIZE);

  SHA1_Init(FALSE, &context);
  memcpy(buf, cred_key, sizeof(nasd_key_t));
  SHA1_Update(FALSE, &context, buf, NASD_HASH_BASIC_SIZE);
  SHA1_Final(FALSE, tdigest, &context);
  bcopy(tdigest, priv_key, NASD_MIN(sizeof(tdigest), sizeof(nasd_key_t)));

  
  return (NASD_SUCCESS);
}

