/*
 * nasd_decompose.c
 *
 * Decompose a NASD id.
 *
 * Gets its own file for debugging purposes.
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_drive_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_freelist.h>
#include <nasd/nasd_itypes.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_cache.h>
#include <nasd/nasd_common.h>

/*
 * Reserved nodes:
 * 0 = null
 * 1 = drive info
 * 2 = partition info
 * 3 = partition objects
 * 4 = trace info
 * 5 = trace data
 *
 * (also update nasd_control.idl if this is updated)
 */
nasd_nodenum_t nasd_reserved_nodes = 6;

nasd_status_t
nasd_od_decompose_control(
  nasd_identifier_t   in_nid,
  nasd_nodenum_t     *nodenump)
{
  nasd_uint64 n64;

  if (in_nid == NASDID_NULL)
    return(NASD_BAD_IDENTIFIER);

  *nodenump = in_nid&0xffffffffUL;
  n64 = in_nid&(~0xffffffffUL);
  if (n64)
    return(NASD_FAIL);
  if ((*nodenump) >= nasd_reserved_nodes)
    return(NASD_FAIL);

  return(NASD_SUCCESS);
}

nasd_status_t
nasd_od_decompose_id(
  nasd_identifier_t   in_nid,
  int                *partnump,
  nasd_nodenum_t     *nodenump,
  nasd_blkno_t       *lvl2_hintp,
  nasd_generation_t  *genp)
{
  nasd_uint64 n64;

  if (in_nid == NASDID_NULL)
    return(NASD_BAD_IDENTIFIER);

  *nodenump = in_nid&0xffffffff;

  n64 = in_nid & nasd_uint64cast(0xfe00000000000000);
  n64 >>= 57;
  n64 &= 0x7f;
  *genp = (nasd_generation_t)n64;

  n64 = in_nid & nasd_uint64cast(0x01e0000000000000);
  n64 >>= 53;
  n64 &= 0xf;
  *partnump = (int)n64;

  n64 = in_nid & nasd_uint64cast(0x000003ff00000000);
  n64 >>= 32;
  n64 &= 0x3ff;
  *lvl2_hintp = (nasd_blkno_t)n64;

  return(NASD_SUCCESS);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
