/*
 * nasd_dux_drive.h
 *
 * Digital Unix specific drive code
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_DUX_KERNEL_DRIVE_H_
#define _NASD__NASD_DUX_KERNEL_DRIVE_H_

#include <vm/vm_page.h>
#include <kern/lock.h>

/*
 * Maximum number _possible_- actual limit is nasd_drive_io_max_coalesce.
 */
#define NASD_IO_MAX_COALESCE 128 /* XXX is this right? */

#define nasd_odc_io_ent_swap_bufs(_ent1_,_ent2_) { \
  vm_page_t _tmp_pp; \
  void  *_tmp_buf; \
  _tmp_buf = (_ent1_)->data.buf; \
  _tmp_pp = (_ent1_)->pp; \
  (_ent1_)->data.buf = (_ent2_)->data.buf; \
  (_ent1_)->pp = (_ent2_)->pp; \
  (_ent2_)->data.buf = _tmp_buf; \
  (_ent2_)->pp = _tmp_pp; \
}

#define NASD_ODC_IO_ENT_CLEAR_PAGE(_ent_) { \
  (_ent_)->data.buf = NULL; \
  (_ent_)->pp = VM_PAGE_NULL; \
}

#define NASD_ODC_IO_ENT_ASSERT_NO_PAGE(_ent_) { \
  NASD_ASSERT((_ent_)->data.buf == NULL); \
  NASD_ASSERT((_ent_)->pp == VM_PAGE_NULL); \
}

/*
 * from nasd_od_kio.c
 */
nasd_status_t nasd_od_k_init(nasd_od_config_t *config);
void nasd_od_k_iodone(struct buf *bp);
nasd_status_t nasd_od_k_blkio(nasd_odc_ent_t *ent, nasd_blkno_t sectno,
  void *buf, int len, int iodir);
nasd_status_t nasd_od_k_blkio_cb(nasd_odc_ent_t *ent, nasd_blkno_t sectno,
  void *buf, int len, int iodir, void (*cb)(void *), void *arg);
nasd_status_t nasd_od_k_write_header(nasd_odc_ent_t *fake_ent,
  nasd_blkno_t sectno, nasd_od_disk_t *disk);
void nasd_psrv_thread(nasd_threadarg_t arg);
int nasd_psrv_go(struct nasd_svinfo *svinfo);
void nasd_od_k_finish_flush(struct buf *bp);
nasd_status_t nasd_od_io_launch_internal(nasd_odc_ent_t *entlist);

#endif /* !_NASD__NASD_DUX_KERNEL_DRIVE_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
