/*
 * nasd_dux_platform.h
 */
/*
 * Copyright (c) 1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_DUX_PLATFORM_H_
#define _NASD__NASD_DUX_PLATFORM_H_

#ifdef KERNEL
#include <track_idle.h>
#include <nasd_server.h>
#ifdef NASD_SECURE_RPCS_ENABLE
#undef NASD_SECURE_RPCS_ENABLE
#endif /* NASD_SECURE_RPCS_ENABLE */
#include <nasd_secure_rpcs_enable.h>
#include <sys/utctime.h>
#else /* KERNEL */
#include <timers.h>
#if NASD_CMU_PDL > 0
#include <pdllib.h>
#endif /* NASD_CMU_PDL > 0 */
#endif /* KERNEL */

#define nasd_int64cast(_n_)   _n_##L
#define nasd_uint64cast(_n_)  _n_##UL
#define NASD_64s_FMT          "ld"
#define NASD_64u_FMT          "lu"
#define NASD_64x_FMT          "lx"
#define NASD_64X_FMT          "lX"
#define NASD_ID_FMT           "016lx"

typedef const void * nasd_qsort_arg_type_t;

#define NASD_INLINE __inline

#ifndef KERNEL

/*
 * I'm so bloody glad that sys/ioctl.h prototypes ioctl
 * _OUTSIDE_ the multiple inclusion protection under dux.
 * FREAKS.
 */
#ifdef __GNUC__
extern int ioctl(int, int, ...);
#endif /* __GNUC__ */

extern void sync(void);

#if NASD_CMU_PDL > 0
#define NASD_IDLE_SUPPORT 1
#endif /* NASD_CMU_PDL > 0 */

#endif /* !KERNEL */

#ifdef KERNEL

#if TRACK_IDLE > 0
#define NASD_IDLE_SUPPORT 1
#endif /* TRACK_IDLE > 0 */

#define NASD_USE_DUX_RW_LOCKS 1

#endif /* KERNEL */

#endif /* !_NASD__NASD_DUX_PLATFORM_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
