/*
 * nasd_cheops_common.h
 */
/*
 * Common CHEOPS definitions
 *
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Khalil Amiri, CMU SCS/ECE, July 18 1997
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
#ifndef _NASD_CHEOPS_COMMON_H_
#define _NASD_CHEOPS_COMMON_H_

#include <dce/rpc.h>
#include <stdio.h>
#include <malloc.h>

#include <nasd/nasd_types.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_pdrive_client.h>
#include <nasd/nasd_threadstuff.h>
#include <nasd/nasd_cheops_types.h>

#ifdef KERNEL

#define     cheops_Msg printf)(

#else /* KERNEL */

#ifdef CHEOPS_PRINT_ERROR 
#define       cheops_Msg                     fprintf)(stderr,
#else /* CHEOPS_PRINT_ERROR */
#define	      cheops_Msg		          fprintf)(stderr,
#endif /* CHEOPS_PRINT ERROR */

#ifndef TRUE
#define TRUE (1)
#define FALSE (0)
#endif /* TRUE */

#endif /* KERNEL */

        typedef char * _nasd_cheops_buffer_t;

        /* cheops object attributes */
        typedef struct _nasd_cheops_attribute_s {
          nasd_uint32            block_size;                               /*   4 bytes */
          nasd_uint64            object_len;                               /*   8 bytes */
          _NASD_CHEOPS_raid_level_t raid_level;
          nasd_identifier_t      ni[_NASD_CHEOPS_MAX_STRIPE_SIZE];         /*   8*8=64 bytes */
          nasd_disk_ident_t      di[_NASD_CHEOPS_MAX_STRIPE_SIZE];         /*   8*8=64 bytes */
          nasd_timespec_t        attr_modify_time;                         /*   8 bytes */
          nasd_timespec_t        object_modify_time;                       /*   8 bytes */
          nasd_timespec_t        object_create_time;                       /*   8 bytes */         
        } _nasd_cheops_attribute_t;
  
	/* cheops drive handle */
	typedef struct _nasd_cheops_dr_handle_s {
	  int			 dr_id;
	  int			 partnum;
          char                   dr_name[_NASD_CHEOPS_DR_NAMELEN];
          char                   dr_port[_NASD_CHEOPS_DR_NAMELEN];
          nasd_drive_handle_t    h;   /* communication handle - user level rpcs */
          int                    fd;  /* fd for kernel pseudo device - kernel level rpcs */
	} _nasd_cheops_dr_handle_t;

	/* cheops storage manager handle */
	typedef struct _nasd_cheops_mgr_handle_s {
	  int		         mgr_id;
          char                   *mgr_name;
          char                   *mgr_port;
          handle_t   h;
        } _nasd_cheops_mgr_handle_t;

	typedef struct _nasd_cheops_mgr_handles_s {
	  int 		   num_managers;
	  _nasd_cheops_mgr_handle_t *hl;
	} _nasd_cheops_mgr_handles_t;

#define MAX_NUM_MANAGERS 8
#define MAX_NUM_DRIVES   16
	
/* error codes */
#define _NASD_CHEOPS_ERR_INIT_FAILED            5000
#define _NASD_CHEOPS_ERR_CANT_CREATE_FILE       5001
#define _NASD_CHEOPS_ERR_ISEMPTY       	   5002
#define _NASD_CHEOPS_ERR_ISLOCKED       	   5003
#define _NASD_CHEOPS_ERR_NOTFOUND       	   5004
#define _NASD_CHEOPS_ERR_READFAILED     	   5005
#define _NASD_CHEOPS_ERR_READINCOMP    	   5006
#define _NASD_CHEOPS_ERR_WRITEFAILED        5007
#define _NASD_CHEOPS_ERR_WRITEINCOMP    	   5008
#define _NASD_CHEOPS_ERR_DOESNOTEXIST       5009

#endif /* !_NASD_CHEOPS_COMMON_H_ */

