/*
 * nasd_common.h
 *
 * shared header file for NASD entities - basic
 * prototypes for library functions
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_COMMON_H_
#define _NASD__NASD_COMMON_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <sys/time.h>

extern char *nasd_error_string(nasd_status_t err);
extern void nasd_printstat(nasd_attribute_t *attr);
extern nasd_status_t nasd_str_to_nasd_id(char *str,
  nasd_identifier_t *nidp);

extern void nasd_gettime(nasd_timespec_t *tm);
extern nasd_status_t nasd_sys_init(void);
extern nasd_status_t nasd_get_clock_speed(nasd_uint64 *ticksp);

extern char *_nasd_timestr(time_t t);
extern char *_nasd_timestr_r(time_t, char *);
extern char *nasd_timestr_r(nasd_timespec_t ts, char *str);

#ifndef KERNEL
#include <netdb.h>

nasd_status_t nasd_gethostbyname_r(char *hostname, struct hostent *hep);
nasd_status_t nasd_raw_disk_len(int fd, nasd_uint64 *lenp);
#endif /* !KERNEL */

#if NASD_IDLE_SUPPORT > 0
extern nasd_status_t nasd_get_total_idle_time(nasd_timespec_t *ts);
#endif /* NASD_IDLE_SUPPORT > 0 */

#endif /* !_NASD__NASD_COMMON_H_ */
