/*
 * nasd_general.h
 *
 * Common NASD defines
 */
/*
 * Copyright (c) 1995,1996,1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: Mark Holland, Daniel Stodolsky, Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD_GENERAL_H_
#define _NASD_GENERAL_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>

#define NASD_TRUE  1
#define NASD_FALSE 0

#define NASD_NSEC_PER_SEC 1000000000
#define NASD_USEC_PER_SEC 1000000
#define NASD_TIMESPEC_VALID(_ts_) (((_ts_).ts_sec >= 0) && ((_ts_).ts_nsec >= 0) && ((_ts_).ts_nsec < NASD_NSEC_PER_SEC))
#define NASD_TIMESPEC_VALID_NZ(_ts_) (((_ts_).ts_sec >= 0) && ((_ts_).ts_nsec >= 0) && ((_ts_).ts_nsec < NASD_NSEC_PER_SEC) && ((_ts_).ts_sec || (_ts_).ts_nsec))
#define NASD_TIMESPEC_EQ(_ts1_,_ts2_) (((_ts1_).ts_sec == (_ts2_).ts_sec) && ((_ts1_).ts_nsec == (_ts2_).ts_nsec))
#define NASD_TIMESPEC_GT(_ts1_,_ts2_) (((_ts1_).ts_sec > (_ts2_).ts_sec) || (((_ts1_).ts_sec == (_ts2_).ts_sec) && ((_ts1_).ts_nsec > (_ts2_).ts_nsec)))
#define NASD_TIMESPEC_LT(_ts1_,_ts2_) (((_ts1_).ts_sec < (_ts2_).ts_sec) || (((_ts1_).ts_sec == (_ts2_).ts_sec) && ((_ts1_).ts_nsec < (_ts2_).ts_nsec)))
#define NASD_TIMESPEC_GE(_ts1_,_ts2_) (((_ts1_).ts_sec > (_ts2_).ts_sec) || (((_ts1_).ts_sec == (_ts2_).ts_sec) && ((_ts1_).ts_nsec >= (_ts2_).ts_nsec)))
#define NASD_TIMESPEC_LE(_ts1_,_ts2_) (((_ts1_).ts_sec < (_ts2_).ts_sec) || (((_ts1_).ts_sec == (_ts2_).ts_sec) && ((_ts1_).ts_nsec <= (_ts2_).ts_nsec)))
#define NASD_TIMESPEC_SUB(_result_,_subtrahend_) { \
  if ((_result_).ts_nsec >= (_subtrahend_).ts_nsec) { \
    (_result_).ts_nsec -= (_subtrahend_).ts_nsec; \
    (_result_).ts_sec -= (_subtrahend_).ts_sec; \
  } \
  else { \
    (_result_).ts_nsec += NASD_NSEC_PER_SEC; \
    (_result_).ts_nsec -= (_subtrahend_).ts_nsec; \
    (_result_).ts_sec -= (_subtrahend_).ts_sec + 1; \
  } \
}
#define NASD_TIMESPEC_ADD(_result_,_addend_) { \
  (_result_).ts_nsec += (_addend_).ts_nsec; \
  (_result_).ts_sec += (_addend_).ts_sec; \
  if ((_result_).ts_nsec >= NASD_NSEC_PER_SEC) { \
    (_result_).ts_nsec -= NASD_NSEC_PER_SEC; \
    (_result_).ts_sec ++; \
  } \
}
#define NASD_TIMESPEC_ZERO(_ts_) { \
  (_ts_).ts_sec = 0; \
  (_ts_).ts_nsec = 0; \
}
#define NASD_TIMESPEC_EQ_ZERO(_ts_) (((_ts_).ts_sec == 0) && ((_ts_).ts_nsec == 0))

#define NASD_CKPART(_a_,_b_,_part_) (((_a_)._part_ != (_b_)._part_) ? (_a_)._part_ - (_b_)._part_ : 
#define NASD_EQPART(_a_,_b_,_part_) (((_a_)._part_ != (_b_)._part_) ? 0 :
#define NASD_COOKIE_CMP(_a_,_b_) (NASD_CKPART(_a_,_b_,part1) NASD_CKPART(_a_,_b_,part2) NASD_CKPART(_a_,_b_,part3) NASD_CKPART(_a_,_b_,part4) NASD_CKPART(_a_,_b_,part5) NASD_CKPART(_a_,_b_,part6) 0)))))))
#define NASD_COOKIE_EQ(_a_,_b_) (NASD_EQPART(_a_,_b_,keydesc.ni) NASD_EQPART(_a_,_b_,keydesc.expiration_seconds) NASD_EQPART(_a_,_b_,keydesc.rights) NASD_EQPART(_a_,_b_,keydesc.min_protection) NASD_EQPART(_a_,_b_,keydesc.drive_name) NASD_EQPART(_a_,_b_,keydesc.userid) NASD_EQPART(_a_,_b_,keydesc.type) 1))))))))

#define NASD_ID_CMP(_a_,_b_) ((_a_) - (_b_))
#define NASD_ID_EQ(_a_,_b_) ((_a_) == (_b_))

#if !defined(KERNEL) && NASD_NOASSERT == 0
#include <assert.h>
#endif /* !KERNEL && NASD_NOASSERT == 0 */

#ifndef KERNEL

#include <stdio.h>

#if NASD_NOASSERT == 0
#define NASD_ASSERT(_x_) { \
  if (!(_x_)) { \
    printf("Assertion check at file %s line %d\n", __FILE__, __LINE__); \
    printf("Assertion failed: %s\n", NASD_STRING(_x_)); \
    fflush(stdout); \
    abort(); \
    exit(1); \
  } \
}
#else /* NASD_NOASSERT == 0 */
#define NASD_ASSERT(x)   {/*noop*/}
#endif /* NASD_NOASSERT == 0 */
#define NASD_PANIC() { \
  printf("D'oh! Fatal error file %s line %d\n", __FILE__, __LINE__); \
  fflush(stdout); \
  fprintf(stderr, "D'oh! Fatal error file %s line %d\n", __FILE__, __LINE__); \
  fflush(stderr); \
  abort(); \
  exit(1); \
}

#else /* !KERNEL */

#ifdef NASD_ASSERT
#undef NASD_ASSERT
#endif /* NASD_ASSERT */

#define NASD_PANIC() { \
  sprintf(nasd_panicbuf, "Fatal NASD error file %s line %d", \
    __FILE__, __LINE__); \
  printf("%s\n", nasd_panicbuf); \
  panic(nasd_panicbuf); \
}

#define NASD_PANICBUF_LEN 2048
extern char nasd_panicbuf[NASD_PANICBUF_LEN];

#if NASD_NOASSERT == 0
#define NASD_ASSERT(_x_) { \
  if (!(_x_)) { \
    sprintf(nasd_panicbuf, \
        "NASD error at line %d file %s (failed asserting %s)", \
        __LINE__, __FILE__, #_x_); \
    printf("%s\n", nasd_panicbuf); \
    thread_block(); \
    panic(nasd_panicbuf); \
  } \
}
#else /* NASD_NOASSERT == 0 */
#define NASD_ASSERT(x) {/*noop*/}
#endif /* NASD_NOASSERT == 0 */

#endif /* !KERNEL */

#ifdef __STDC__
#define NASD_STRING(_str_) #_str_
#else /* __STDC__ */
#define NASD_STRING(_str_) "_str_"
#endif /* __STDC__ */

#define NASD_MIN(_a_,_b_) (((_a_)<(_b_))?(_a_):(_b_))
#define NASD_MAX(_a_,_b_) (((_a_)>(_b_))?(_a_):(_b_))
#define NASD_ABS(_a_)     (((_a_) > 0) ? (_a_) : (-(_a_)))

#define NASD_ATOMIC_TIMESPEC_ADD(_resultp_,_addendp_) \
  nasd_atomic_add_timespec(_resultp_,_addendp_)

#ifdef DEC_OSF
extern unsigned long nasd_atomic_add_timespec(
  nasd_timespec_t *result, nasd_timespec_t *addend);
#endif /* DEC_OSF */

#ifndef DEC_OSF
extern void nasd_atomic_add_timespec(nasd_timespec_t *result,
  nasd_timespec_t *addend);
#endif /* !DEC_OSF */

#if defined(DEC_OSF) && defined(KERNEL)

#define NASD_ATOMIC_INC64(_var_) atomic_incq((_var_))
#define NASD_ATOMIC_DEC64(_var_) atomic_decq((_var_))
#define NASD_ATOMIC_ADD64(_var_,_val_) atomic_addq((_var_),(_val_))

#else /* DEC_OSF && KERNEL */

#define NASD_ATOMIC_INC64(_var_) nasd_atomic_inc64((_var_))
#define NASD_ATOMIC_DEC64(_var_) nasd_atomic_add64((_var_), (-1))
#define NASD_ATOMIC_ADD64(_var_,_val_) nasd_atomic_add64((_var_),(_val_))
extern void nasd_atomic_inc64(nasd_int64 *);
extern void nasd_atomic_add64(nasd_int64 *, nasd_int64);
extern void nasd_atomic_sub64(nasd_int64 *, nasd_int64);

#endif /* DEC_OSF && KERNEL */

#ifdef KERNEL
#ifndef NULL
#define NULL 0L
#endif /* !NULL */
#endif /* KERNEL */

#endif /* _NASD_GENERAL_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
