/*
 * nasd_mq.h
 *
 * Cheesy message-queue and shared memory interface to a NASD drive.
 *
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Marc Unangst
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD_MQ_H_
#define _NASD_MQ_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_pdrive_client.h>

typedef struct NASDMQ_null_msg_s {
  int seqnum;
  int replyq;
} NASDMQ_null_msg_t;

typedef struct NASDMQ_null_rep_s {
  nasd_status_t nasd_status;
  nasd_rpc_status_t op_status;
} NASDMQ_null_rep_t;

typedef struct NASDMQ_sync_msg_s {
  int seqnum;
  int replyq;
} NASDMQ_sync_msg_t;

typedef struct NASDMQ_sync_rep_s {
  nasd_status_t nasd_status;
  error_status_t op_status;
} NASDMQ_sync_rep_t;

typedef struct NASDMQ_p_crpart_msg_s {
  int seqnum;
  int replyq;
  nasd_key_t in_key;
  nasd_p_part_creat_dr_args_t args;
} NASDMQ_p_crpart_msg_t;

typedef struct NASDMQ_p_crpart_rep_s {
  nasd_p_part_creat_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_crpart_rep_t;

typedef struct NASDMQ_p_create_msg_s {
  int seqnum;
  int replyq;
  nasd_key_t in_key;
  nasd_p_create_dr_args_t args;
} NASDMQ_p_create_msg_t;

typedef struct NASDMQ_p_create_rep_s {
  nasd_p_create_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_create_rep_t;

typedef struct NASDMQ_p_getattr_msg_s {
  int seqnum;
  int replyq;
  nasd_key_t in_key;
  nasd_p_getattr_dr_args_t args;
} NASDMQ_p_getattr_msg_t;

typedef struct NASDMQ_p_getattr_rep_s {
  nasd_p_getattr_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_getattr_rep_t;

typedef struct NASDMQ_p_write_simple_msg_s {
  int seqnum;
  int replyq;
  int shmid;
  nasd_key_t in_key;
  nasd_p_smpl_op_dr_args_t args;
} NASDMQ_p_write_simple_msg_t;

typedef struct NASDMQ_p_write_simple_rep_s {
  nasd_p_fastwrite_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_write_simple_rep_t;

typedef struct NASDMQ_p_read_simple_msg_s {
  int seqnum;
  int replyq;
  int shmid;
  nasd_key_t in_key;
  int is_read2;
  nasd_p_smpl_op_dr_args_t args;
} NASDMQ_p_read_simple_msg_t;

typedef struct NASDMQ_p_read_simple_rep_s {
  nasd_p_fastread_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_read_simple_rep_t;

typedef struct NASDMQ_p_tread_simple_msg_s {
  int seqnum;
  int replyq;
  int shmid;
  nasd_key_t in_key;
  nasd_p_thrtl_op_dr_args_t args;
} NASDMQ_p_tread_simple_msg_t;

typedef struct NASDMQ_p_tread_simple_rep_s {
  nasd_p_fastread_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_tread_simple_rep_t;

typedef struct NASDMQ_p_setattr_msg_s {
  int seqnum;
  int replyq;
  nasd_key_t in_key;
  nasd_p_setattr_dr_args_t args;
} NASDMQ_p_setattr_msg_t;

typedef struct NASDMQ_p_setattr_rep_s {
  nasd_p_setattr_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_setattr_rep_t;

typedef struct NASDMQ_p_flush_obj_msg_s {
  int seqnum;
  int replyq;
  nasd_key_t in_key;
  nasd_p_flush_obj_dr_args_t args;
} NASDMQ_p_flush_obj_msg_t;

typedef struct NASDMQ_p_flush_obj_rep_s {
  nasd_p_flush_obj_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_flush_obj_rep_t;

typedef struct NASDMQ_p_eject_obj_msg_s {
  int seqnum;
  int replyq;
  nasd_key_t in_key;
  nasd_p_eject_obj_dr_args_t args;
} NASDMQ_p_eject_obj_msg_t;

typedef struct NASDMQ_p_eject_obj_rep_s {
  nasd_p_eject_obj_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_eject_obj_rep_t;

typedef struct NASDMQ_p_remove_msg_s {
  int seqnum;
  int replyq;
  nasd_key_t in_key;
  nasd_p_remove_dr_args_t args;
} NASDMQ_p_remove_msg_t;

typedef struct NASDMQ_p_remove_rep_s {
  nasd_p_remove_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_remove_rep_t;

typedef struct NASDMQ_p_initialize_msg_s {
  int seqnum;
  int replyq;
  nasd_p_initialize_dr_args_t args;
} NASDMQ_p_initialize_msg_t;

typedef struct NASDMQ_p_initialize_rep_s {
  nasd_p_initialize_dr_res_t res;
  nasd_rpc_status_t op_status;
} NASDMQ_p_initialize_rep_t;


#define NASD_MSGQ_KEY		0x4e415344 /* NASD */

/* largest possible message */
/* should be largest of:
    8+(largest ARGS_MAX struct)
    12+(largest RES_MAX struct) */
#define NASD_MSGQ_MSGLEN	(8+NASD_P_SETATTR_DR_ARGS_MAX)

/* message types */
#define NASD_MSGQ_BOGUS		0
#define NASD_MSGQ_NULL		1
#define NASD_MSGQ_SYNC		2
#define NASD_MSGQ_PCRPART	3
#define NASD_MSGQ_PCREATE	4
#define NASD_MSGQ_PGETATTR	5
#define NASD_MSGQ_PWRITE_SIMPLE	6
#define NASD_MSGQ_PREAD_SIMPLE	7
#define NASD_MSGQ_PTREAD_SIMPLE	8
#define NASD_MSGQ_PSETATTR	9
#define NASD_MSGQ_PFLUSH_OBJ	10
#define NASD_MSGQ_PEJECT_OBJ	11
#define NASD_MSGQ_PREMOVE	12
#define NASD_MSGQ_PINITIALIZE	13

typedef struct nasd_msgq_buf_s {
  long mtype;
  union {
    NASDMQ_null_msg_t null_msg;
    NASDMQ_null_rep_t null_rep;
    NASDMQ_sync_msg_t sync_msg;
    NASDMQ_sync_rep_t sync_rep;
    NASDMQ_p_crpart_msg_t p_crpart_msg;
    NASDMQ_p_crpart_rep_t p_crpart_rep;
    NASDMQ_p_create_msg_t p_create_msg;
    NASDMQ_p_create_rep_t p_create_rep;
    NASDMQ_p_getattr_msg_t p_getattr_msg;
    NASDMQ_p_getattr_rep_t p_getattr_rep;
    NASDMQ_p_write_simple_msg_t p_write_simple_msg;
    NASDMQ_p_write_simple_rep_t p_write_simple_rep;
    NASDMQ_p_read_simple_msg_t p_read_simple_msg;
    NASDMQ_p_read_simple_rep_t p_read_simple_rep;
    NASDMQ_p_tread_simple_msg_t p_tread_simple_msg;
    NASDMQ_p_tread_simple_rep_t p_tread_simple_rep;
    NASDMQ_p_setattr_msg_t p_setattr_msg;
    NASDMQ_p_setattr_rep_t p_setattr_rep;
    NASDMQ_p_flush_obj_msg_t p_flush_obj_msg;
    NASDMQ_p_flush_obj_rep_t p_flush_obj_rep;
    NASDMQ_p_eject_obj_msg_t p_eject_obj_msg;
    NASDMQ_p_eject_obj_rep_t p_eject_obj_rep;
    NASDMQ_p_remove_msg_t p_remove_msg;
    NASDMQ_p_remove_rep_t p_remove_rep;
    NASDMQ_p_initialize_msg_t p_initialize_msg;
    NASDMQ_p_initialize_rep_t p_initialize_rep;
    char mtext[NASD_MSGQ_MSGLEN];
  } un;
} nasd_msgq_buf_t;

nasd_status_t nasd_startup_msgq(void);
nasd_status_t nasd_od_mq_init(void);
void nasd_od_mq_shutdown(void);
void nasd_od_mq_rm_msgq(void *arg);

#endif /* !_NASD_MQ_H_ */
