/*
 * nasd_options.h
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Authors: Howard Gobioff, Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_OPTIONS_H_
#define _NASD__NASD_OPTIONS_H_

/*
 * Enable security
 *
 * Check if it's already defined- we may have something on the
 * command-line as a result of the user explicitly requesting
 * security-yes or security-no from itomf. (Note: in all current
 * ports of NASD, there is a way of altering this value that
 * is preferred to/supercedes the setting here.)
 */
#ifndef NASD_SECURE_RPCS_ENABLE
#define NASD_SECURE_RPCS_ENABLE  0
#endif /* !NASD_SECURE_RPCS_ENABLE */

/* Use the plain HMAC for processing  - This is primarily to compare
what performance is like without the overhead of the hiearchichal MAC.
This forces the client/receiver to just HMAC all data exchanged */
#define NASD_USE_HMAC_ONLY 0

/* debug atomic ops (locks, etc) */
#define NASD_DEBUG_ATOMIC 0

/* show shutdown tasks as they occur */
#define NASD_DEBUG_SHUTDOWN 0

/* track outstanding memory allocation count */
#define NASD_MEM_COUNT_ALLOC 1

/* print freelist statistics during shutdown */
#define NASD_FREELIST_STATS 0

/* find core leaks -- EXPENSIVE */
#define NASD_TRACK_ALLOCATIONS 0

/* NFS keep stats on cache ops */
#define NASD_NFS_KEEP_CACHE_STATS 1

/* NFS report cache ops stats */
#define NASD_NFS_DUMP_CACHE_STATS 0

/* Testing scaffold for timing mechanism */
#define NASD_TIMEOUT_TEST 0

/* Some security stats  */
#define NASD_SECURITY_KEEP_STATS  0
#define NASD_SECURITY_PRINT_STATS 0

/*
 * Set this value >=3 to attempt to detect corruption by array
 * over or underwrites.
 */
#define NASD_MEM_FENCEPOST 0

/*
 * Set this to a 32-bit fill pattern, eg 0xdeadbeef
 */
#define NASD_MEM_FILL 0

#if defined(MACH) && defined(OSF) && defined(KERNEL)
#define DEC_OSF        1
#ifdef ASSEMBLER
#ifdef __LANGUAGE_C__
#undef __LANGUAGE_C__
#endif /* __LANGUAGE_C__ */
#else /* ASSEMBLER */
#define __LANGUAGE_C__ 1
#endif /* ASSEMBLER */
#endif /* MACH && OSF && KERNEL */

#if defined(__alpha) && defined(__osf__)
#ifndef DEC_OSF
#define DEC_OSF 1
#endif /* !DEC_OSF */
#endif /* __alpha && __osf__ */

#ifdef __linux__
#ifndef LINUX
#define LINUX 1
#endif /* !LINUX */
#endif /* __linux__ */

#if defined(VXWORKS)
#define NASD_WIND_THREADS 1
#endif /* VXWORKS */

#if 0
#define db_printf(_a_) printf _a_
#else
#define db_printf(_a_) { /* noop */ }
#endif

#include <nasd/nasd_site.h>

#endif /* !_NASD__NASD_OPTIONS_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
