/*
 * nasd_pdev.h
 *
 * pseudo-device for performing nasd operations in the kernel on
 * behalf of a user-process
 *
 * Intended for use by AFS server, which can't make DCE RPC calls
 * for itself.
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD_KPDEV_NASD_KPDEV_H_
#define _NASD_KPDEV_NASD_KPDEV_H_

#include <nasd/nasd_options.h>
#include <sys/ioctl.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_pdrive_client.h>

typedef struct NASDPD_bind_s {
  char     drive_name[256];
  char     drive_port[256];
  int      binding_type;
  caddr_t  binding_param;
  int      binding_param_len;
} NASDPD_bind_t;

typedef struct NASDPD_null_s {
  unsigned int        start_cc;
  unsigned int        stop_cc;
  nasd_status_t       nasd_status;
  nasd_rpc_status_t   op_status;
} NASDPD_null_t;

typedef struct NASDPD_sync_s {
  unsigned int    start_cc;
  unsigned int    stop_cc;
  nasd_status_t   nasd_status;
  error_status_t  op_status;
} NASDPD_sync_t;

typedef struct NASDPD_p_crpart_s {
  unsigned int                 start_cc;
  unsigned int                 stop_cc;
  nasd_key_t                   in_key;
  nasd_p_part_creat_dr_args_t  args;
  nasd_p_part_creat_dr_res_t   res;
  nasd_rpc_status_t            op_status;
} NASDPD_p_crpart_t;

typedef struct NASDPD_p_create_s {
  unsigned int              start_cc;
  unsigned int              stop_cc;
  nasd_key_t                in_key;
  nasd_p_create_dr_args_t  *args;
  nasd_p_create_dr_res_t   *res;
  nasd_rpc_status_t         op_status;
} NASDPD_p_create_t;

typedef struct NASDPD_p_getattr_s {
  unsigned int               start_cc;
  unsigned int               stop_cc;
  nasd_key_t                 in_key;
  nasd_p_getattr_dr_args_t  *args;
  nasd_p_getattr_dr_res_t   *res;
  nasd_rpc_status_t          op_status;
} NASDPD_p_getattr_t;

typedef struct NASDPD_p_write_simple_s {
  unsigned int                start_cc;
  unsigned int                stop_cc;
  nasd_key_t                  in_key;
  void                       *buf;
  nasd_p_smpl_op_dr_args_t    args;
  nasd_p_fastwrite_dr_res_t   res;
  nasd_rpc_status_t           op_status;
} NASDPD_p_write_simple_t;

typedef struct NASDPD_p_read_simple_s {
  unsigned int               start_cc;
  unsigned int               stop_cc;
  nasd_key_t                 in_key;
  int                        is_read2;
  void                      *buf;
  nasd_p_smpl_op_dr_args_t   args;
  nasd_p_fastread_dr_res_t   res;
  nasd_rpc_status_t          op_status;
} NASDPD_p_read_simple_t;

typedef struct NASDPD_p_tread_simple_s {
  unsigned int                start_cc;
  unsigned int                stop_cc;
  nasd_key_t                  in_key;
  void                       *buf;
  nasd_p_thrtl_op_dr_args_t   args;
  nasd_p_fastread_dr_res_t    res;
  nasd_rpc_status_t           op_status;
} NASDPD_p_tread_simple_t;

typedef struct NASDPD_p_setattr_s {
  unsigned int               start_cc;
  unsigned int               stop_cc;
  nasd_key_t                 in_key;
  nasd_p_setattr_dr_args_t  *args;
  nasd_p_setattr_dr_res_t   *res;
  nasd_rpc_status_t          op_status;
} NASDPD_p_setattr_t;

typedef struct NASDPD_p_flush_obj_s {
  unsigned int                start_cc;
  unsigned int                stop_cc;
  nasd_key_t                  in_key;
  nasd_p_flush_obj_dr_args_t  args;
  nasd_p_flush_obj_dr_res_t   res;
  nasd_rpc_status_t           op_status;
} NASDPD_p_flush_obj_t;

typedef struct NASDPD_p_eject_obj_s {
  unsigned int                start_cc;
  unsigned int                stop_cc;
  nasd_key_t                  in_key;
  nasd_p_eject_obj_dr_args_t  args;
  nasd_p_eject_obj_dr_res_t   res;
  nasd_rpc_status_t           op_status;
} NASDPD_p_eject_obj_t;

typedef struct NASDPD_p_remove_s {
  unsigned int             start_cc;
  unsigned int             stop_cc;
  nasd_key_t               in_key;
  nasd_p_remove_dr_args_t  args;
  nasd_p_remove_dr_res_t   res;
  nasd_rpc_status_t        op_status;
} NASDPD_p_remove_t;

typedef struct NASDPD_p_initialize_s {
  unsigned int                 start_cc;
  unsigned int                 stop_cc;
  nasd_p_initialize_dr_args_t  args;
  nasd_p_initialize_dr_res_t   res;
  nasd_rpc_status_t            op_status;
} NASDPD_p_initialize_t;

typedef struct NASDPD_t_timeout_test_s {
  nasd_status_t    nasd_status;
  nasd_timespec_t  delta;
} NASDPD_t_timeout_test_t;

#define NASDPD_PBIND        _IOW ('N', 20, NASDPD_bind_t)
#define NASDPD_PUNBIND      _IO  ('N', 21)

#define NASDPD_PNULL        _IOWR('N', 25, NASDPD_null_t)
#define NASDPD_PSYNC        _IOWR('N', 26, NASDPD_sync_t)

#define NASDPD_P_PCRPART        _IOWR('N', 50, NASDPD_p_crpart_t)
#define NASDPD_P_PCREATE        _IOWR('N', 51, NASDPD_p_create_t)
#define NASDPD_P_PGETATTR       _IOWR('N', 52, NASDPD_p_getattr_t)
#define NASDPD_P_PWRITE_SIMPLE  _IOWR('N', 53, NASDPD_p_write_simple_t)
#define NASDPD_P_PREAD_SIMPLE   _IOWR('N', 54, NASDPD_p_read_simple_t)
#define NASDPD_P_PTREAD_SIMPLE  _IOWR('N', 55, NASDPD_p_tread_simple_t)
#define NASDPD_P_PSETATTR       _IOWR('N', 56, NASDPD_p_setattr_t)
#define NASDPD_P_PFLUSH_OBJ     _IOWR('N', 57, NASDPD_p_flush_obj_t)
#define NASDPD_P_PREMOVE        _IOWR('N', 58, NASDPD_p_remove_t)
#define NASDPD_P_PINITIALIZE    _IOWR('N', 59, NASDPD_p_initialize_t)
#define NASDPD_P_PEJECT_OBJ     _IOWR('N', 60, NASDPD_p_eject_obj_t)
#define NASDPD_P_PWRITE_FAKE    _IOWR('N', 61, NASDPD_p_write_simple_t)
#define NASDPD_P_PREAD_FAKE     _IOWR('N', 62, NASDPD_p_read_simple_t)

#define NASDPD_T_TIMEOUT_TEST   _IOWR('T', 1, NASDPD_t_timeout_test_t)

#define NASDPD_AOP_WBUF_MAX 2048

#endif /* !_NASD_KPDEV_NASD_KPDEV_H_ */
