/*
 * nasd_security_dr.h
 *
 * Put the Security in NASD *DRIVE* !
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD_SECURITY_DR_H_
#define _NASD_SECURITY_DR_H_ 

#include <nasd/nasd_security_types.h>
#include <nasd/nasd_security.h>
#include <nasd/nasd_itypes.h>
#include <nasd/nasd_cache.h>


  

void nasd_security_dr_print_capability(nasd_key_t in_key, nasd_capability_args_t* cap_args);

nasd_status_t nasd_security_verify_args(nasd_args_t *in_args,
					nasd_byte_t *in_args_otw,
					nasd_args_len_t in_args_len,
					nasd_byte_t *in_data,
					nasd_int32 out_in_len,
					nasd_digest_nonce_otw_t in_digest,
                                        nasd_key_t in_key,
                                        nasd_security_context_t* contextp);



nasd_status_t nasd_security_digest_res(nasd_args_t *in_args,
                                       nasd_key_t in_key,
				       nasd_byte_t *out_res_otw,
				       nasd_args_len_t out_res_len,
				       nasd_byte_t *out_data,
				       nasd_int32 out_data_len,
                                       nasd_security_context_t *contextp,
				       nasd_digest_nonce_otw_t out_digest);


nasd_status_t nasd_security_get_request_key(nasd_args_t *in_args,
                                            nasd_byte_t *in_args_otw,
                                            nasd_args_len_t in_args_len,
                                            nasd_key_t out_key);			 


void nasd_security_reply_init_dr(nasd_args_t *in_args,
                                 nasd_key_t in_key,
                                 nasd_byte_t* icv,
                                 nasd_security_context_t *security_context,
                                 nasd_offset_t start_offset);


/* in drive/nasd_security_dr.c */
void nasd_security_context_update_send_dr(nasd_security_context_t *context,
                                       nasd_byte_t *buf, nasd_len_t len,
                                       nasd_blkrec_t* blkp, nasd_int32 use_zero);

nasd_status_t nasd_security_context_update_receive_dr(nasd_security_context_t *sec_ctx,
                                                      nasd_byte_t *data,
                                                      nasd_len_t len, 
                                                      nasd_procpipe_t *byte_pipe,
                                                      nasd_len_t remain,
                                                      nasd_blkrec_t *blkrec,
                                                      nasd_timespec_t *pipe_stall) ;


#define NASD_SECURITY_CACHE_BUCKETS 128
#define NASD_SECURITY_CACHE_ELEMENTS_DEFAULT 512 

/* Structures, Macros, and functions to manage the on drive capability cache */

#define NASD_SECURITY_NI_HASH(_ni_) (((_ni_)&0xffffffff)%NASD_SECURITY_CACHE_BUCKETS)

#define NASD_CURRENT_DAILY_KEY_CACHE 1
#define NASD_PREVIOUS_DAILY_KEY_CACHE 2
#define NASD_NO_KEY_CACHE 0

/* in drive/nasd_security_dr.c */
nasd_status_t nasd_security_cache_invalidate(nasd_partnum_t partnum, nasd_identifier_t ni);

nasd_status_t nasd_security_cache_lookup(nasd_capability_args_t *cap_args, nasd_key_t out_key);

nasd_status_t nasd_security_cache_insert(nasd_capability_args_t *cap_args, nasd_key_t in_key);

nasd_status_t nasd_security_init_dr();
nasd_status_t nasd_security_cache_init();
nasd_status_t nasd_security_cache_new_daily_key();

/* in common/nasd_password.c */
nasd_status_t nasd_security_password_to_keys(char *password,
					     nasd_key_t master_key,
					     nasd_key_t drive_key,
					     nasd_partnum_t partnum,
					     nasd_key_t part_key,
					     nasd_key_t red_key,
					     nasd_key_t black_key);

#endif /* !_NASD_SECURITY_DR_H_ */
