/*
 * nasd_shell.h
 *
 * NASD-shell definitions shared across shell and
 * loadable modules
 */
/*
 * Copyright (c) 1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_SHELL_H_
#define _NASD__NASD_SHELL_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_shutdown.h>
#include <nasd/nasd_varargs.h>

#define NASD_SHELL_MAJOR_VERS 1
#define NASD_SHELL_MINOR_VERS 0

typedef struct nasd_shell_action_s    nasd_shell_action_t;
typedef struct nasd_shell_variable_s  nasd_shell_variable_t;

typedef nasd_status_t (*nasd_shell_action_proc_t)(
  char  **args,
  int     nargs);

#define NASD_SHELL_VAR_TYPE_STRING    0

#define NASD_SHELL_VAR_FLAG_NONE      0
#define NASD_SHELL_VAR_FLAG_RO    (1<<0) /* readonly */

struct nasd_shell_action_s {
  char                      *name;
  nasd_shell_action_proc_t   proc;
};

struct nasd_shell_variable_s {
  char                      *name;
  int                        type;
  int                        flags;
  nasd_shell_action_proc_t   change_proc;
  void                      *addr;
};

extern void nasd_shell_shutdown(int code);
extern nasd_status_t nasd_shell_add_var(nasd_shell_variable_t *var);
extern nasd_status_t nasd_shell_add_action(nasd_shell_action_t *act);

extern void nasd_shell_warn NASD_VARARGS_PROTO((char *fmt, ...));
extern void nasd_shell_output NASD_VARARGS_PROTO((char *fmt, ...));
extern void nasd_shell_flush_output(void);

#endif /* !_NASD__NASD_SHELL_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
