/*
 * nasd_cache.h
 *
 * Structures for cache for on-disk NASD filesystem
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD_SHUTDOWN_H_
#define _NASD_SHUTDOWN_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_threadstuff.h>

#if defined (DEC_OSF) && defined(KERNEL)
#include <vm/vm_page.h>
#endif /* DEC_OSF && KERNEL */

typedef struct nasd_shutdown_s         nasd_shutdown_t;
typedef struct nasd_shutdown_list_s    nasd_shutdown_list_t;

/*
 * nasd_shutdown
 *
 * Entry in a shutdown list.
 * If proc is NULL, it's a memory entry.
 */
struct nasd_shutdown_s {
  void             (*proc)(void *arg);
  void              *arg;
  int                size;
  nasd_shutdown_t   *next;
  char              *file;
  int                line;
};

/*
 * nasd_shutdown_list
 *
 * List of shutdowns.
 */
struct nasd_shutdown_list_s {
  NASD_DECLARE_MUTEX(lock)
  nasd_shutdown_t       *list;
  nasd_shutdown_list_t  *next;
};

#define nasd_shutdown_proc(_sl_, _proc_, _arg_) \
  _nasd_shutdown_proc(_sl_, _proc_, _arg_, __FILE__, __LINE__)

#define nasd_shutdown_mem(_sl_, _addr_, _size_) \
  _nasd_shutdown_mem(_sl_, _addr_, _size_, __FILE__, __LINE__)

/*
 * from nasd_shutdown.c
 */
nasd_status_t nasd_shutdown_sys_init(void);
nasd_status_t nasd_shutdown_list_init(nasd_shutdown_list_t **slp);
nasd_status_t nasd_shutdown_list_shutdown_internal(nasd_shutdown_list_t *sl,
  int verbose);
nasd_status_t nasd_shutdown_list_shutdown(nasd_shutdown_list_t *sl);
nasd_status_t nasd_shutdown_list_shutdown_verbose(nasd_shutdown_list_t *sl);
nasd_status_t _nasd_shutdown_proc(nasd_shutdown_list_t *sl,
  void (*proc)(void *), void *arg, char *file, int line);
nasd_status_t _nasd_shutdown_mem(nasd_shutdown_list_t *sl,
  void *addr, int size, char *file, int line);
void nasd_shutdown_cleanup(void);
void nasd_shutdown_mutex(void *arg);
void nasd_shutdown_cond(void *arg);
void nasd_shutdown_rwlock(void *arg);
#if defined(DEC_OSF) && defined(KERNEL)
void nasd_shutdown_page(void *arg);
#endif /* DEC_OSF && KERNEL */
extern void nasd_shutdown_shutdown(void *arg);

#endif /* !_NASD_SHUTDOWN_H_ */
