/*
 * nasd_sys.h
 *
 * Kernel interface for in-kernel NASD drive.
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_SYS_H_
#define _NASD__NASD_SYS_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <sys/param.h>

typedef struct nasd_od_config_s nasd_od_config_t;
typedef struct nasd_options_s   nasd_options_t;

struct nasd_od_config_s {
  nasd_layout_type_t   layout_type;
  nasd_ioqueue_type_t  ioqueue_type;
  int                  ios_outstanding;
};

struct nasd_serv {
  char               root_path[MAXPATHLEN];
  dev_t              dev;
  int                do_format;
  int                cache_blocks;
  int                svc_threads;
  int                stack_size;
  int                port;
  int                verbose;
  int                use_tcp;
  int                security_enabled;
  nasd_uint64        max_disk_len;
  nasd_uint16        required_protection;
  nasd_drive_name_t  drive_name;
  nasd_key_t         daily;
  nasd_key_t         old_daily;
  nasd_key_t         working;
  nasd_key_t         master;
  nasd_od_config_t   config;
};

struct nasd_nfs_mount {
  char         drive_ip_addr[16];
  char         name[MAXHOSTNAMELEN];
  char         path[MAXPATHLEN];
  int          portnum;
  int          partnum;
  nasd_uint32  net_addr;
};

struct nasd_nfs_config {
  struct nasd_nfs_mount  *mount_array;
  int                     mount_array_len;
  int                     verbose;
  int                     svc_threads;
  int                     stack_size;
  int                     colocate;
};

struct nasd_options_s {
  nasd_uint64  opts1;
};

/* opts1 stuff */
#define NASD_OPTS1_SECURITY   (1<<0)
#define NASD_OPTS1_NFS_CLIENT (1<<1)
#define NASD_OPTS1_NFS_FM     (1<<2)
#define NASD_OPTS1_DRIVE      (1<<3)

#ifdef KERNEL
struct nasd_svinfo {
  struct nasd_serv   srv;
  int                became_listener;
  size_t             size;            /* pdrive only */
};

struct nasd_nfs_svinfo {
  struct nasd_nfs_config  config;
  int                     became_listener;
};
#endif /* KERNEL */

#define NASD_GET_OPTIONS  1
/*                        2 unused */
#define NASD_PSRV_GO      3
#define NASD_SRV_PANIC    4
#define NASD_NFS_SRV_GO   5

#define NASD_WIRE_DIR_READ  1
#define NASD_WIRE_DIR_WRITE 2
#define NASD_WIRE_DIR_TREAD 3

#if defined(DEC_OSF) && defined(KERNEL)
extern lock_data_t nasd_d_lock;
extern struct lockinfo *nasd_d_lockinfo;

#define NASD_D_LOCK()        lock_write(&nasd_d_lock)
#define NASD_D_READ_LOCK()   lock_read(&nasd_d_lock)
#define NASD_D_UNLOCK()      lock_done(&nasd_d_lock)
#define NASD_D_LOCK_INIT()   lock_setup(&nasd_d_lock,nasd_d_lockinfo,TRUE)
#define NASD_D_LOCK_HOLDER() (LOCK_HOLDER(&nasd_d_lock))
#endif /* DEC_OSF && KERNEL */

#ifndef KERNEL
extern int nasd_srv(int cmd, void *arg);
#endif /* !KERNEL */

#ifdef DEC_OSF
extern int nasd_wire_buf(void *buf, int len, void **iobufp, int dir);
extern int nasd_unwire_buf(void *buf, int len, void *iobuf, int dir);
#endif /* DEC_OSF */

#endif /* !_NASD__NASD_SYS_H_ */
