/*
 * atomic_tester.c
 *
 * Test various atomic ops
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_common.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_general.h>

#include <stdio.h>

char *progname;

extern char *optarg;
extern int   optind;

main(
  int     argc,
  char  **argv)
{
  nasd_timespec_t ts1, ts2, ts3, ts4;
  nasd_uint64 a, b, c;
  int failed;

  failed = 0;

  a = 1;
  b = 2;
  c = 3;

  NASD_ATOMIC_INC64(&a);
  if (a != 2) {
    fprintf(stderr, "ERROR: failed NASD_ATOMIC_INC64\n");
    fprintf(stderr, "  got a=%" NASD_64u_FMT " expected 2\n", a);
    failed++;
  }

  NASD_ATOMIC_ADD64(&c, b);
  if (c != 5) {
    fprintf(stderr, "ERROR: failed NASD_ATOMIC_ADD64\n");
    fprintf(stderr, "  got c=%" NASD_64u_FMT " expected 5\n", c);
    failed++;
  }

#if 0
  NASD_ATOMIC_SUB64(&b, a);
  if (b != 0) {
    fprintf(stderr, "ERROR: failed NASD_ATOMIC_SUB64\n");
    fprintf(stderr, "  got b=%" NASD_64u_FMT " expected 0\n", b);
    failed++;
  }
#endif

  NASD_ATOMIC_DEC64(&c);
  if (c != 4) {
    fprintf(stderr, "ERROR: failed NASD_ATOMIC_DEC64\n");
    fprintf(stderr, "  got c=%" NASD_64u_FMT " expected 0\n", c);
    failed++;
  }

  ts1.ts_sec  = 12345;
  ts1.ts_nsec = 123456789;
  ts2.ts_sec  = 65432;
  ts2.ts_nsec = 543210000;

  NASD_ATOMIC_TIMESPEC_ADD(&ts1, &ts2);
  if ((ts1.ts_sec != 77777) || (ts1.ts_nsec != 666666789)
    || (ts2.ts_sec != 65432) || (ts2.ts_nsec != 543210000))
  {
    fprintf(stderr, "ERROR: failed NASD_ATOMIC_TIMESPEC_ADD\n");
    fprintf(stderr, "  got ts1=%d:%09d ts2=%d:%09d\n",
      ts1.ts_sec, ts1.ts_nsec, ts2.ts_sec, ts2.ts_nsec);
    fprintf(stderr, "  expected ts1=77777:666666789 ts2=65432:543210000\n");
  }

  ts1.ts_sec  = 5;
  ts1.ts_nsec = 999999999;
  ts2.ts_sec  = 1;
  ts2.ts_nsec = 2;

  NASD_ATOMIC_TIMESPEC_ADD(&ts1, &ts2);
  if ((ts1.ts_sec != 7) || (ts1.ts_nsec != 1)
    || (ts2.ts_sec != 1) || (ts2.ts_nsec != 2))
  {
    fprintf(stderr, "ERROR: failed NASD_ATOMIC_TIMESPEC_ADD\n");
    fprintf(stderr, "  got ts1=%d:%09d ts2=%d:%09d\n",
      ts1.ts_sec, ts1.ts_nsec, ts2.ts_sec, ts2.ts_nsec);
    fprintf(stderr, "  expected ts1=7:000000001 ts2=1:000000002\n");
    failed++;
  }

  ts2.ts_sec  = 5;
  ts2.ts_nsec = 6;

  NASD_ATOMIC_TIMESPEC_ADD(&ts1, &ts2);
  if ((ts1.ts_sec != 12) || (ts1.ts_nsec != 7)
    || (ts2.ts_sec != 5) || (ts2.ts_nsec != 6))
  {
    fprintf(stderr, "ERROR: failed NASD_ATOMIC_TIMESPEC_ADD\n");
    fprintf(stderr, "  got ts1=%d:%09d ts2=%d:%09d\n",
      ts1.ts_sec, ts1.ts_nsec, ts2.ts_sec, ts2.ts_nsec);
    fprintf(stderr, "  expected ts1=12:000000007 ts2=5:000000006\n");
    failed++;
  }

  if (failed)
    exit(1);

  printf("Passed simple tests of atomic ops\n");

  exit(0);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
