/*
 * pcrpart.c
 *
 * Create partition on a NASD drive
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <nasd/nasd_pdrive.h>
#include <nasd/nasd_pdrive_client.h>

char *progname;

nasd_error_string_t error_text;

void
usage()
{
  fprintf(stderr, "USAGE: %s servername  master_password\n", progname);
  fflush(stderr);
  exit(1);
}

int
main(argc, argv)
  int     argc;
  char  **argv;
{
  nasd_p_initialize_dr_args_t args;
  nasd_p_initialize_dr_res_t res;
  nasd_drive_handle_t h;
  nasd_rpc_status_t status;
  int  nblocks;
  nasd_partnum_t partnum=1;
  nasd_key_t red_key,black_key,part_key;
  char *server_name;
  nasd_status_t rc;
  int sec_level=0;
  progname = argv[0];

  if (argc != 3)
    usage();

  server_name = argv[1];
  
  rc = nasd_cl_p_init();
  if (rc) {
    fprintf(stderr,
      "ERROR: cannot initialize client library, error 0x%x (%s)\n",
      rc, nasd_error_string(rc));
    fflush(stderr);
    exit(1);
  }

  rc = nasd_bind_to_drive(server_name, NASD_PDRIVE_PORT,
    NASD_BIND_DCE_DIRECT, NULL, 0, &h);
  if (rc) {
    fprintf(stderr, "ERROR: cannot bind to server %s error 0x%x (%s)\n",
      server_name, rc, nasd_error_string(rc));
    fflush(stderr);
    exit(1);
  }

  nasd_security_password_to_keys(argv[2],args.in_master_key,args.in_drive_key,
    partnum,part_key,red_key,black_key);
  bzero((char *)&res, sizeof(nasd_p_initialize_dr_res_t));
  nasd_cl_p_initialize_dr(h, &args, &res, &status);

  if (res.nasd_status || status) {
    fprintf(stderr, "ERROR: nasd_status 0x%x (%s) status 0x%x (%s)\n",
      res.nasd_status, nasd_error_string(res.nasd_status),
      status, nasd_cl_error_string(h, status, error_text));
    exit(2);
  }

  printf("Succeeded\n");
  fflush(stdout);

  rc = nasd_unbind_drive(&h);
  if (rc) {
    fprintf(stderr, "ERROR: got 0x%x (%s) from nasd_unbind_drive()\n",
      rc, nasd_error_string(rc));
    fflush(stderr);
    exit(2);
  }

  nasd_cl_p_shutdown();

  exit(0);
}

