/*
 * premove.c
 *
 * Remove object from NASD drive.
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <nasd/nasd_pdrive.h>
#include <nasd/nasd_pdrive_client.h>
#include <nasd/nasd_security.h>
#include <getopt.h>
#include <string.h>

char *progname;

extern char *optarg;
extern int   optind;

nasd_error_string_t error_text;
nasd_key_t part_key,master_key,drive_key,red_key,black_key;


void
usage()
{
  fprintf(stderr, "USAGE: %s servername partnum nasdid master_password\n", progname);
  fflush(stderr);
  exit(1);
}

int
main(argc, argv)
  int     argc;
  char  **argv;
{
  nasd_p_remove_dr_args_t args;
  nasd_p_remove_dr_res_t res;
  nasd_rpc_status_t status;
  nasd_drive_handle_t h;
  char *server_name;
  nasd_status_t rc;
  int p;

  progname = argv[0];

  if (argc != 5)
    usage();

  server_name = argv[1];
  bzero((char *)&args, sizeof(args));

  if (sscanf(argv[2], "%d", &p) != 1)
    usage();
  args.in_partnum = p;

  rc = nasd_str_to_nasd_id(argv[3], &args.in_identifier);
  if (rc)
    usage();

  rc = nasd_cl_p_init();
  if (rc) {
    fprintf(stderr,
      "ERROR: cannot initialize client library, error 0x%x (%s)\n",
      rc, nasd_error_string(rc));
    fflush(stderr);
    exit(1);
  }

  rc = nasd_bind_to_drive(server_name, NASD_PDRIVE_PORT,
    NASD_BIND_DCE_DIRECT, NULL, 0, &h);
  if (rc) {
    fprintf(stderr, "ERROR: cannot bind to server %s error 0x%x (%s)\n",
      server_name, rc, nasd_error_string(rc));
    fflush(stderr);
    exit(1);
  }

  nasd_security_password_to_keys(argv[4],master_key,drive_key,p,part_key,red_key,black_key);

  /* Fill in capability and security param */ 
  args.in_security_param.type = NASD_BLACK_KEY;
  args.in_security_param.actual_protection = NASD_SECURE_OPS;
  args.in_security_param.partnum=p;
  nasd_cl_p_remove_dr(h, black_key,&args, &res, &status);

  if (res.nasd_status || status) {
    fprintf(stderr, "nasd_status=0x%x (%s) status=0x%x (%s)\n",
      res.nasd_status, nasd_error_string(res.nasd_status),
      status, nasd_cl_error_string(h, status, error_text));
    exit(1);
  }

  nasd_cl_p_shutdown();

  printf("Succeeded.\n");

  exit(0);
}
