/*
 * timeout.c
 *
 * Test front-end for generic NASD timeout mechanism
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_timeout.h>
#include <nasd/nasd_pdev.h>
#include <nasd/nasd_mem.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

char *progname;

extern char *optarg;
extern int   optind;

#define DEF_NTIMES 1
int ntimes = DEF_NTIMES;

void
usage()
{
  fprintf(stderr, "USAGE: %s [options]\n", progname);
  fprintf(stderr, "Options:\n");
  fprintf(stderr, "  -n niterations [default %d]\n", DEF_NTIMES);
  fflush(stderr);
  exit(1);
}

int
main(
  int     argc,
  char  **argv)
{
  nasd_timespec_t ts;
  nasd_status_t rc;
  char c;
  int i;

  progname = argv[0];

  while ((c = getopt(argc, argv, "n:")) != EOF) {
    switch(c) {
      case 'n':
        if (sscanf(optarg, "%d", &ntimes) != 1)
          usage();
        break;
      default:
        fprintf(stderr, "Unknown option '%c'\n", c);
        usage();
    }
  }

  if (optind < argc)
    usage();

  if (ntimes < 1)
    usage();

  rc = nasd_threads_init();
  if (rc) {
    fprintf(stderr, "ERROR: got 0x%x (%s) from nasd_threads_init()\n",
      rc, nasd_error_string(rc));
    fflush(stderr);
    exit(1);
  }

  for(i=0;i<ntimes;i++) {
    nasd_gettime(&ts);
    printf("%d:%09d  (#%d)\n", ts.ts_sec, ts.ts_nsec, i);
  }

  nasd_threads_shutdown();

  exit(0);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
