/*
 * timeout.c
 *
 * Test front-end for generic NASD timeout mechanism
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_timeout.h>
#include <nasd/nasd_pdev.h>
#include <nasd/nasd_mem.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

char *progname;

extern char *optarg;
extern int   optind;

int use_kernel = 0;

nasd_timespec_t delta = { 0, 1000000 };

#if NASD_TIMEOUT_TEST == 0
nasd_status_t
nasd_timeout_test()
{
  return(NASD_OP_NOT_SUPPORTED);
}
#endif /* NASD_TIMEOUT_TEST == 0 */

void
usage()
{
  fprintf(stderr, "USAGE: %s [options]\n", progname);
  fprintf(stderr, "Options:\n");
  fprintf(stderr, "  -d delta (seconds, decimals allowed)\n");
  fprintf(stderr, "  -k use kernel device\n");
  fflush(stderr);
  exit(1);
}

int
main(
  int     argc,
  char  **argv)
{
  NASDPD_t_timeout_test_t tt;
  nasd_status_t rc;
  double dd, d2;
  char *devname;
  int fd, ret;
  char c;

  progname = argv[0];

  while ((c = getopt(argc, argv, "d:k")) != EOF) {
    switch(c) {
      case 'd':
        if (sscanf(optarg, "%lf", &dd) != 1)
          usage();
        if (dd < (double)0.0)
          usage();
        delta.ts_sec = (nasd_int32)dd;
        d2 = dd - (double)delta.ts_sec;
        d2 *= (double)NASD_NSEC_PER_SEC;
        delta.ts_nsec = (nasd_int32)d2;
        break;
      case 'k':
        use_kernel = 1;
        break;
      default:
        fprintf(stderr, "Unknown option '%c'\n", c);
        usage();
    }
  }

  if (optind < argc)
    usage();

  if (use_kernel == 0) {
    rc = nasd_timeout_test(delta);
  }
  else {
    devname = "/dev/nasdkp0";
    fd = open(devname, O_RDWR);
    if (fd < 0) {
      perror(devname);
      fprintf(stderr, "ERROR: could not open %s\n", devname);
      exit(1);
    }
    bzero((char *)&tt, sizeof(tt));
    tt.delta = delta;
    ret = ioctl(fd, NASDPD_T_TIMEOUT_TEST, &tt);
    if (ret) {
      perror("NASDPD_T_TIMEOUT_TEST");
      fprintf(stderr, "ERROR: could not perform test ioctl\n");
      exit(1);
    }
    rc = tt.nasd_status;
    close(fd);
  }

  if (rc) {
    fprintf(stderr, "ERROR: failed timeout test with 0x%x (%s)\n",
      rc, nasd_error_string(rc));
    exit(1);
  }

  printf("Passed timeout test\n");

  fflush(stderr);
  fflush(stdout);

  exit(0);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
