/*
 *  protos.h -- function prototypes.
 *
 *  Copyright 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

/*  main.c  */
void setsource(char *line, int echo);	/* process cpp file/line info */
void statement(Node *tree);		/* process one statement */

void warning(char *msg, char *detail);	/* issue warning message */
void fatal(char *msg, char *detail);	/* issue fatal error message */
void tfatal(Nodeptr n, char*m, char*d);	/* issue fatal err for line n->n_lno */
void twarning(Nodeptr n,char*m,char*d);	/* issue warning for line n->n_lno */

void malf(char *file, int ln, char *s);	/* abort due to malfunction */
void *zalloc(long b);			/* call calloc() with error checking */

/*  annot.c  */
Annot mkannot(int tsz, int msz, int algn, Order *o);  /* make annotation */
int setannot(Type *t, Annot a);		/* set annotation of a declarator */
int okalign(Annot a);			/* check that alignment is power of 2 */
Oseg ointerval(int start, int end);	/* make order segment */
Order *ostart(Oseg g);			/* start accumulating ordering info */
Order *oappend(Oseg g);			/* append more ordering info */
Order *oclone(Order *o);		/* allocate copy of ordering struct */
void bitorder(Symptr sym, int size, Order *o);	/* set bitfield ordering */
int boff(Order *o, int offset);		/* return offset of nth-LSB */
Mdef *mattrib(Mdef *old, Symbol *asg);	/* update mdef with attribute value */
void pannot(Type *t);			/* print annotation on stderr */

/*  fgrain.c  */
void gfgrain(Frame *f, int la, int ra);	/* generate code using chunking alg */

/*  gen.c  */
void genheader(Symbol *func);		/* generate function/macro header */
void genstmt(Node *tree);		/* generate code for statement */
void gentail(Symbol *func);		/* generate end of function/macro */

/*  gframe.c  */
Frame *gframes(Type *lt, Type *rt);	/* generate frames for assignment */
Frame *sortframes(Frame *flist,int maj,int min);  /* sort on offset & shift */

/*  grammar.y  */
int yyparse();				/* parse USC file */

/*  naive.c  */
void gnaive(Frame *f, int la, int ra);	/* generate naive code from framelist */

/*  out.c */				
void out(char *fmt, ...);		/* write code to stdout and/or pfile */

/*  symtab.c  */
char *savename(char *name);		/* save unique copy of char string */
Symbol *lookup(char *name, int kind);	/* find/create symbol table entry */
Symbol *defsym(char *name, Scode sc);	/* define a symbol as type sc */
Symptr mappend(Symptr old, Symptr new);	/* append new members to old list */
void dumpsyms(void);			/* dump symbol table */
void rmparms(void);			/* remove parameter declarations */

/*  tokens.c  */
void initkw(void);			/* initialize keyword table */
char *yystring(void);			/* current lex token, as string */

/*  tree.c  */
Nodeptr newnode(int tok, Nodeptr l, Nodeptr r);  /* create new node */
void ptree(Nodeptr n);			/* print node tree stderr */
void freenodes(void);			/* free all currently allocated nodes */
char *tokname(int tok);			/* return name of token code */

/*  type.c  */
void inittypes(void);			/* initialize predefined types */
Type *newtype(Tcode tc, Type *utype);	/* alloc and init type structure */
Type *tclone(Type *t);			/* make a copy of a type structure */
Type *tmerge(Type *old, Type *mod);	/* merge attribs of "mod" into "old" */
Type *tfix(Type *t);			/* clean up loose ends of type decl */
Symptr setdecl(Node *n,Type *t,Annot a);/* set declarator type from parse tree*/
void ptype(Type *t, int xtdef);		/* print type synopsis on stderr */
Symptr mktag(Type *t);			/* invent a struct/union tag */
void functype(Type *t);			/* check function return type */
int okparm(Type *t);			/* is type valid as a parameter? */
int okmember(Type *t);			/* is type valid as a struct member? */
void typecheck(Nodeptr tree);		/* check & set types in tree */
int sametype(Type *ltype, Type *rtype);	/* are types structurally identical? */

/*  wordg.c  */
void gwords(Frame *f, int la, int ra);	/* generate word-based moves */
