/*
 *  usc.h -- master #include file for USC.
 *
 *  Copyright 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#ifdef LINUX
#include <getopt.h>
#endif /* LINUX */

#include "structs.h"		/* internal data structures */
#include "protos.h"		/* function prototypes */
#include "tokens.h"		/* token codes */


#define VERSION "USC version 2.1"

#define DEF_GENERATOR gwords;	/* default code generator function */


#define BYTE 8			/* number of bits in a byte */

#define MAXINTBYTES 32		/* largest allowed int, in bytes (power of 2) */
#define MAXINTBITS (BYTE * MAXINTBYTES)


/* debugging flags */
#define DB_YACC		'y'	/* yacc tokens */
#define DB_PARSE	'p'	/* raw parse tree */
#define DB_TREE		't'	/* typed parse tree */
#define DB_LINES	'l'	/* annotate trees with line numbers */
#define DB_SYMTAB	's'	/* symbol table */

#define DB_MORSELS	'm'	/* morsels */
#define DB_IFRAMES	'i'	/* individual frames before combination */
#define DB_UFRAMES	'u'	/* unsorted frames */
#define DB_SFRAMES	'v'	/* sorted frames */
#define DB_FRAMES	'f'	/* final frame list */
#define DB_BITOPS	'b'	/* destination bit operations */

#define DB_ALL		'a'	/* all values except 'y' */
#define DB_GENERAL	'g'	/* generally useful values */
#define DB_G_STR	"tlsf"	/* values used by DB_GENERAL */


/* macros for loading or storing via an address and an offset */
#define IntAt(base,offset) (*(int*)((char*)base+offset))
#define CharAt(base,offset) (*(char*)((char*)base+offset))

/* macro for diagnosing a program bug */
#define MALFUNC(msg) malf(__FILE__, __LINE__, msg)


/* global variables */
#include "globals.h"

#ifdef LINUX
extern int yyerror(char *);
extern int yylex(void);
#endif /* LINUX */
